import { Pulse } from './pulse';
import { Tracker } from './internal';
interface ActionContext {
    trackers: Set<Tracker>;
    errorHandlers: (false | ((e: unknown) => unknown))[];
}
declare type Modifiers = ReturnType<typeof actionMods>;
export declare type FuncType = (modifiers: Modifiers, ...args: any) => any;
export declare type HigherOrderFunc<F extends FuncType> = F extends (modifiers: Modifiers, ...args: infer P) => ReturnType<F> ? (...args: P) => ReturnType<F> : never;
/**
 * @class
 * Pulse Action
 */
export declare class Action<T extends FuncType = FuncType> {
    instance: () => Pulse;
    private action;
    name: string;
    constructor(instance: () => Pulse, action: T);
    /**
     * @public
     * Return the higher order function with the correct types & context
     */
    func(): HigherOrderFunc<T>;
    /**
     * @internal
     * The higher order function
     */
    private _func;
    /**
     * @internal
     * The higher order function
     */
    private _funcSync;
}
declare function actionMods(context: ActionContext): {
    onCatch: (...callbacks: (false | ((e: unknown) => unknown))[]) => void;
    undo: () => void;
    /**
     * @public
     * This creates a tracker bound to the execution context, can be used several times in a single action.
     */
    batch: (func: () => unknown) => void;
    /**
     * @public
     * This creates a tracker bound to the execution context, can be used several times in a single action.
     */
    track: (func: () => unknown) => Tracker;
    uncaught: (e?: unknown) => void;
};
export {};
//# sourceMappingURL=action.d.ts.map