"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Action = void 0;
const internal_1 = require("./internal");
/**
 * @class
 * Pulse Action
 */
class Action {
    constructor(instance, action) {
        this.instance = instance;
        this.action = action;
    }
    /**
     * @public
     * Return the higher order function with the correct types & context
     */
    func() {
        if (this.action.constructor.name === 'AsyncFunction')
            return this._func.bind(this);
        else if (this.action.constructor.name === 'Function')
            return this._funcSync.bind(this);
        else
            console.warn('This is not a function');
    }
    /**
     * @internal
     * The higher order function
     */
    _func() {
        return __awaiter(this, arguments, void 0, function* () {
            const context = {
                trackers: new Set(),
                errorHandlers: []
            };
            try {
                // invoke the function and supply the modifiers
                return yield this.action(actionMods.call(this, context), ...arguments);
            }
            catch (e) {
                let returnFalse = false;
                // on error, run the error callbacks
                for (const [index, callback] of context.errorHandlers.entries()) {
                    if (typeof callback == 'boolean') {
                        returnFalse = true;
                        continue;
                    }
                    if (index == context.errorHandlers.length - 1)
                        return callback(e);
                    else
                        callback(e);
                }
                if (returnFalse)
                    return false;
            }
            finally {
                context.trackers.forEach(tracker => tracker.destroy());
            }
        });
    }
    /**
     * @internal
     * The higher order function
     */
    _funcSync() {
        const context = {
            trackers: new Set(),
            errorHandlers: []
        };
        try {
            // invoke the function and supply the modifiers
            return this.action(actionMods.call(this, context), ...arguments);
        }
        catch (e) {
            let returnFalse = false;
            // on error, run the error callbacks
            for (const [index, callback] of context.errorHandlers.entries()) {
                if (typeof callback == 'boolean') {
                    returnFalse = true;
                    continue;
                }
                if (index == context.errorHandlers.length - 1)
                    return callback(e);
                else
                    callback(e);
            }
            if (returnFalse)
                return false;
        }
        finally {
            context.trackers.forEach(tracker => tracker.destroy());
        }
    }
}
exports.Action = Action;
function actionMods(context) {
    return {
        onCatch: (...callbacks) => {
            // call default global error handler
            callbacks.unshift((e) => this.instance().createError(e, { fromAction: this }));
            context.errorHandlers = callbacks;
        },
        // finally: (func: () => unknown) => {},
        undo: () => {
            context.trackers.forEach(tracker => tracker.undo());
        },
        /**
         * @public
         * This creates a tracker bound to the execution context, can be used several times in a single action.
         */
        batch: (func) => {
            this.instance().batch(func);
        },
        /**
         * @public
         * This creates a tracker bound to the execution context, can be used several times in a single action.
         */
        track: (func) => {
            const tracker = new internal_1.Tracker(this.instance, func);
            context.trackers.add(tracker);
            return tracker;
        },
        uncaught: (e) => {
            if (e)
                throw e;
        }
    };
}
//# sourceMappingURL=action.js.map