"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Selector = void 0;
const internal_1 = require("../internal");
class Selector extends internal_1.Computed {
    constructor(collection, key) {
        // initialize computed constructor with initial compute state
        super(collection().instance, () => Selector.findData(collection(), key));
        // this is the selected primary key
        this._id = 0;
        // computed function that returns a given item from collection
        this.func = () => Selector.findData(collection(), this._id);
        // alias collection function
        this.collection = collection;
        this.type(Object);
        this._id = key;
    }
    // getter and setter for primary key
    set id(val) {
        this._id = val;
        this.recompute();
    }
    get id() {
        if (this.instance().runtime.trackState)
            this.instance().runtime.foundState.add(this);
        return this._id;
    }
    select(key) {
        this.id = key;
    }
    // custom override for the State persist function
    persist(key) {
        this.persistState = true;
        this.instance().storage.handleStatePersist(this, key);
        return this;
    }
    getPersistableValue() {
        return this.id;
    }
    static findData(collection, key) {
        if (key == undefined)
            return null;
        let data = collection.getData(key).value;
        // if data is not found, create placeholder data, so that when real data is collected it maintains connection
        if (!data) {
            // this could be improved by storing temp references outside data object in collection
            collection.data[key] = new internal_1.Data(() => collection, { id: key });
            data = collection.getData(key).value;
        }
        else {
            // If we have a computed function, run it before returning the data.
            data = collection._computedFunc ? collection._computedFunc(data) : data;
        }
        return data;
    }
    reset() {
        super.reset();
        this._id = 0;
        return this;
    }
}
exports.Selector = Selector;
exports.default = Selector;
//# sourceMappingURL=selector.js.map