"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Computed = void 0;
const internal_1 = require("./internal");
class Computed extends internal_1.State {
    constructor(instance, func, deps) {
        super(instance, instance().config.computedDefault || null);
        this.instance = instance;
        this.func = func;
        this.deps = deps;
        instance()._computed.add(this);
        if (typeof func !== 'function')
            throw new TypeError('A compute function must be provided to Computed.');
        if (deps)
            deps.forEach(state => state.dep.depend(this));
        // if Core will not be used, or Pulse in a post-core state (ready), compute immediately
        if (instance().config.noCore === true || instance().ready)
            this.recompute();
    }
    // private cleanup: Set<State> = new Set();
    set value(val) {
        console.error('Error: Can not mutate Computed value, please use recompute()');
    }
    get value() {
        return this._value;
    }
    set bind(val) {
        console.error('Error: Can not bind Computed value');
    }
    computeValue() {
        if (this.deps)
            return this.func();
        this.instance().runtime.trackState = true;
        const computed = this.func();
        let dependents = this.instance().runtime.getFoundState();
        dependents.forEach(state => state.dep.depend(this));
        return computed;
    }
    recompute() {
        this.set(this.computeValue());
    }
    reset() {
        super.reset();
        this.recompute();
        return this;
    }
    patch() {
        console.error('Error, can not use patch method on Computed since the value is dynamic.');
        return this;
    }
    persist(key) {
        console.error('Computed state can not be persisted, remove call to .persist()', key);
        return this;
    }
}
exports.Computed = Computed;
exports.default = Computed;
//# sourceMappingURL=computed.js.map