"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Pulse = exports.defaultConfig = void 0;
// prettier-ignore
const internal_1 = require("./internal");
const status_1 = __importDefault(require("./status"));
exports.defaultConfig = {
    noCore: false
};
class Pulse {
    constructor() {
        this.ready = false;
        this.controllers = {};
        this.errorHandlers = new Set();
        this.history = [];
        // Core reference
        this._core = {};
        // Context reference
        this._computed = new Set();
        this._state = new Set();
        this._collections = new Set();
        this.nonce = 0;
        this.config = {};
        this.integrations = new internal_1.Integrations(() => this);
        this.subController = new internal_1.SubController(this);
        this.status = new status_1.default(() => this);
        this.runtime = new internal_1.Runtime(this);
        this.storage = new internal_1.Storage(() => this, this.config.storage || {});
        this.globalBind();
        this.integrations.pulseReady();
    }
    onCoreReady(core) {
        if (!this.ready)
            this.ready = true;
        // Copy core object structure without destroying this.core object reference
        if (core)
            for (let p in core)
                this._core[p] = core[p];
        this._computed.forEach(instance => instance.recompute());
        this.integrations.coreReady();
    }
    core(core) {
        if (core)
            this.onCoreReady(core);
        return this._core;
    }
    with(integration) {
        this.integrations.use(integration);
        return this;
    }
    nextPulse(callback) {
        this.runtime.nextPulse(callback);
    }
    track(changeFunc) {
        return new internal_1.Tracker(() => this, changeFunc);
    }
    batch(batchFunc) {
        this.runtime.batch(batchFunc);
    }
    setStorage(config) {
        const persistedState = this.storage.persistedState;
        this.storage = new internal_1.Storage(() => this, config);
        this.storage.persistedState = persistedState;
        this.storage.persistedState.forEach(state => state.persist(state.name));
    }
    createError(error, info) {
        // handle
    }
    /**
     * Global reference to the first pulse instance created this runtime
     */
    globalBind() {
        try {
            if (!globalThis.__pulse__)
                globalThis.__pulse__ = Pulse;
            if (!globalThis.__pulse__app)
                globalThis.__pulse__app = this;
        }
        catch (error) { } // fail silently
    }
    getNonce() {
        this.nonce++;
        return this.nonce;
    }
    configure(config) {
        this.config = config;
    }
}
exports.Pulse = Pulse;
Pulse.initialIntegrations = [];
exports.default = Pulse;
//# sourceMappingURL=pulse.js.map