export interface PulseResponse<DataType = any> {
    data: DataType;
    timedout?: boolean;
    status: number;
    raw?: Response;
    type?: string;
}
export interface APIConfig {
    options: RequestInit;
    baseURL?: string;
    path?: string;
    timeout?: number;
}
export declare class Route {
    config: APIConfig;
    constructor(config?: APIConfig);
    get(endpoint: string): Promise<PulseResponse<any>>;
    post(endpoint: string, payload?: any): Promise<PulseResponse<any>>;
    put(endpoint: string, payload?: any): Promise<PulseResponse<any>>;
    patch(endpoint: string, payload?: any): Promise<PulseResponse<any>>;
    delete(endpoint: string, payload?: any): Promise<PulseResponse<any>>;
    send(method: string, endpoint: string, payload?: any): Promise<PulseResponse>;
    static cloneDefaultConfig(config: APIConfig): APIConfig;
}
export interface RouteConfig {
    baseURL?: string;
    timeout?: number;
    options?: RequestInit;
}
export interface CallRouteConfig {
    params?: Record<string, any>;
    query?: Record<string, any>;
    body?: Record<string, any>;
    method?: 'GET' | 'PUT' | 'POST' | 'PATCH' | 'DELETE';
    options?: RequestInit;
}
/**
 * @param config.options normal fetch options such as headers, credentials, etc.
 * @param config.baseURL The URL to be used on each request (if left empty, defaults to current hostname)
 * @returns The configured route function
 */
export declare function route(config?: RouteConfig): <ResponseType_1>(path: string, inConfig?: CallRouteConfig) => Promise<PulseResponse<ResponseType_1>>;
//# sourceMappingURL=route.d.ts.map