import { Pulse } from './pulse';
import { State, SubscriptionContainer, Tracker } from './internal';
export interface IJob {
    id?: number;
    state: State;
    newStateValue?: any;
    background?: boolean;
    parentJob?: number;
    jobSpawnedFrom?: Function;
    preventDepIngest?: boolean;
}
export interface JobConfigInterface {
    perform?: boolean;
    background?: boolean;
    jobSpawnedFrom?: Function;
    parentJob?: number;
    batched?: boolean;
}
export declare class Runtime {
    instance: () => Pulse;
    currentJob: IJob | null;
    private jobsQueue;
    private jobsToRerender;
    private tasksOnceComplete;
    trackers: Set<Tracker>;
    trackState: boolean;
    foundState: Set<State>;
    private batchJobs;
    private batchQueue;
    constructor(pulseInstance: Pulse);
    /**
     * @internal
     * Creates a Job out of State and new Value and than add it to a job queue
     */
    ingest(state: State, newStateValue?: any, options?: JobConfigInterface): void;
    /**
     * @internal
     * Perform a State Update
     */
    private perform;
    /**
     * SideEffects are side effects of the perform function.. for instance the watchers
     */
    private sideEffects;
    batch(func: () => unknown): void;
    getUniqueDependentsRecursively(stateItems: State[]): Set<State>;
    /**
     * @internal
     * This will be update all Subscribers of complete jobs
     */
    private updateSubscribers;
    /**
     * @internal
     * Builds an object out of propKeysChanged in the SubscriptionContainer
     */
    formatChangedPropKeys(subscriptionContainer: SubscriptionContainer): {
        [key: string]: any;
    };
    getFoundState(): Set<State<any>>;
    nextPulse(callback: () => any): void;
}
export default Runtime;
//# sourceMappingURL=runtime.d.ts.map