"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.normalizeArray = exports.validateNumber = exports.cleanse = exports.normalizeMap = exports.isAsync = exports.isFunction = exports.genId = exports.defineConfig = exports.shallowmerge = exports.normalizeGroups = exports.copy = exports.normalizeDeps = exports.getPulseInstance = exports.resetState = exports.cleanState = void 0;
const isWatchableObj_1 = require("./helpers/isWatchableObj");
function cleanState(state) {
    return {
        value: state.value,
        previousState: state.previousState,
        isSet: state.isSet,
        dependents: state.dep.deps.size,
        subscribers: state.dep.subs.size,
        name: state.name
    };
}
exports.cleanState = cleanState;
function resetState(stateToReset) {
    const check = (instance) => {
        if (typeof instance.reset === 'function') {
            instance.reset();
            return true;
        }
        else
            return false;
    };
    if (!Array.isArray(stateToReset))
        stateToReset = [stateToReset];
    for (const i in stateToReset) {
        const success = check(stateToReset[i]);
        if (!success && typeof stateToReset[i] === 'object')
            for (const name in stateToReset[i])
                check(stateToReset[i][name]);
    }
}
exports.resetState = resetState;
function getPulseInstance(state) {
    try {
        if (state.instance)
            return state.instance();
        else
            return globalThis.__pulse;
    }
    catch (e) { }
}
exports.getPulseInstance = getPulseInstance;
function normalizeDeps(deps) {
    return Array.isArray(deps) ? deps : [deps];
}
exports.normalizeDeps = normalizeDeps;
exports.copy = val => {
    // ignore if primitive type
    if (typeof val !== 'object' || val instanceof Date)
        return val;
    if (isWatchableObj_1.isWatchableObject(val))
        val = Object.assign({}, val);
    else if (Array.isArray(val))
        val = [...val];
    return val;
};
// groups are defined by the user as an array of strings, this converts them into object/keys
function normalizeGroups(groupsAsArray = []) {
    const groups = {};
    for (let i = 0; i < groupsAsArray.length; i++) {
        const groupName = groupsAsArray[i];
        groups[groupName] = [];
    }
    return groups;
}
exports.normalizeGroups = normalizeGroups;
function shallowmerge(source, changes) {
    let keys = Object.keys(changes);
    keys.forEach(property => {
        source[property] = changes[property];
    });
    return source;
}
exports.shallowmerge = shallowmerge;
function defineConfig(config, defaults) {
    return Object.assign(Object.assign({}, defaults), config);
}
exports.defineConfig = defineConfig;
function genId() {
    return Math.random().toString().split('.')[1] + Date.now();
}
exports.genId = genId;
function isFunction(func) {
    return typeof func === 'function';
}
exports.isFunction = isFunction;
function isAsync(func) {
    return func.constructor.name === 'AsyncFunction';
}
exports.isAsync = isAsync;
function normalizeMap(map) {
    return Array.isArray(map) ? map.map(key => ({ key, val: key })) : Object.keys(map).map(key => ({ key, val: map[key] }));
}
exports.normalizeMap = normalizeMap;
function cleanse(object) {
    if (!isWatchableObj_1.isWatchableObject(object))
        return object;
    const clean = Object.assign({}, object);
    const properties = Object.keys(clean);
    for (let i = 0; i < properties.length; i++) {
        const property = properties[i];
        if (isWatchableObj_1.isWatchableObject(clean[property])) {
            clean[property] = cleanse(clean[property]);
        }
    }
    return clean;
}
exports.cleanse = cleanse;
function validateNumber(mutable, amount) {
    if (typeof amount !== 'number' || typeof mutable !== 'number') {
        return false;
    }
    return true;
}
exports.validateNumber = validateNumber;
function normalizeArray(items) {
    if (items == undefined)
        return [];
    return Array.isArray(items) ? items : [items];
}
exports.normalizeArray = normalizeArray;
//# sourceMappingURL=utils.js.map